import pytest
import os
import numpy as np
import pyscal.core as pc
import pyscal.crystal_structures as pcs


def test_file_system():

    atoms, boxdims = pcs.make_crystal('bcc', repetitions = [1, 1, 1])
    sys = pc.System()
    sys.box = boxdims
    sys.atoms = atoms
    sys.find_neighbors(method = 'voronoi')

    sys.to_file('tests/tjkf.dat')

    #now try to read in the file
    sys2 = pc.System()
    sys2.read_inputfile('tests/tjkf.dat')
    assert len(sys2.atoms) == 2

    #now add some custom values
    atoms[0].custom = {"velocity":12}
    atoms[1].custom = {"velocity":24}

    #now try to read in the file
    sys3 = pc.System()
    sys3.box = boxdims
    sys3.atoms = atoms
    sys3.to_file('tests/tjkf.dat', customkeys=['velocity'])

    #now read it again
    sys4 = pc.System()
    sys4.read_inputfile('tests/tjkf.dat', customkeys=['velocity'])
    #now get atoms and check them
    atoms = sys4.atoms
    assert int(atoms[0].custom['velocity']) == 12
    assert int(atoms[1].custom['velocity']) == 24
    sys4.to_file('tests/tjkf.dat.gz', customkeys=['velocity'],
        compressed=True)

    #now read it again
    sys5 = pc.System()
    sys5.read_inputfile('tests/tjkf.dat.gz', customkeys=['velocity'])
    #now get atoms and check them
    atoms = sys5.atoms
    assert int(atoms[0].custom['velocity']) == 12
    assert int(atoms[1].custom['velocity']) == 24

    sys6 = pc.System()
    sys6.read_inputfile('tests/tjkf.dat')
    sys6.to_file('tests/poscar_1', format="poscar", species=['Mo'])
    sys6.to_file('tests/data_1', format="lammps-data", species=['Mo'])


import pytest
import os
import numpy as np
import pyscal.core as pc
import pyscal.crystal_structures as pcs

def test_complex_system():
    sys = pc.System()
    sys.read_inputfile('examples/cluster.dump')
    sys.find_neighbors(method='cutoff', cutoff=3.63)
    assert 176 == sys.find_solids(bonds=6, threshold=0.5, avgthreshold=0.6, cluster=True)

def test_cluster():
    sys = pc.System()
    sys.read_inputfile('examples/cluster.dump')
    sys.find_neighbors(method='cutoff', cutoff=3.63)
    sys.find_solids(cluster=False)
    val = sys.cluster_atoms("solid", largest = True)
    assert 176 == val

def test_cluster_cutoff():
    sys = pc.System()
    sys.read_inputfile('examples/cluster.dump')
    sys.find_neighbors(method='cutoff', cutoff=3.63)
    sys.find_solids(cluster=False)
    val = sys.cluster_atoms("solid", largest = True, cutoff=3.63)
    assert 176 == val

def test_system_nucsize_fraction():
    #create some atoms
    atoms, boxdims = pcs.make_crystal('bcc', repetitions = [2, 2, 2], lattice_constant=3.20)
    sys = pc.System()
    sys.box = boxdims
    sys.atoms = atoms

    #test that atoms are set properly
    assert len(sys.atoms) == 16

    #now calculate nucsize
    sys.find_neighbors(method='cutoff', cutoff=3.63)
    assert 16 == sys.find_solids(bonds=0.8, threshold=0.5, avgthreshold=0.6, cluster=True)


import pytest
import os
import numpy as np
import pyscal.core as pc


def test_entropy():
	sys = pc.System()
	sys.read_inputfile("tests/conf.fcc.Al.dump")
	sys.find_neighbors(method="cutoff", cutoff=0)
	
	lat = (sys.box[0][0]/5)

	sys.calculate_entropy(1.4*lat, ra=0.9*lat, averaged=True,
		switching_function=True)
	atoms = sys.atoms
	solid_entropy = [atom.entropy for atom in atoms]
	solid_avg_entropy = [atom.avg_entropy for atom in atoms]
	#assert np.mean(solid_entropy) == 2
	assert np.abs(np.mean(solid_entropy) + 3.471) < 0.001
	assert np.abs(np.mean(solid_avg_entropy) + 3.471) < 0.001

	sys.calculate_entropy(1.4*lat, averaged=True)
	atoms = sys.atoms
	solid_entropy = [atom.entropy for atom in atoms]
	solid_avg_entropy = [atom.avg_entropy for atom in atoms]
	#assert np.mean(solid_entropy) == 2
	assert np.abs(np.mean(solid_entropy) + 3.471) < 0.001
	assert np.abs(np.mean(solid_avg_entropy) + 3.471) < 0.001


import pytest
import os
import numpy as np
import pyscal.core as pc
import pyscal.crystal_structures as pcs

def test_neighbors_system():
    #create some atoms
    atoms, boxdims = pcs.make_crystal('bcc', repetitions = [6, 6, 6])
    sys = pc.System()
    sys.box = boxdims
    sys.atoms = atoms

    #test that atoms are set properly
    assert len(sys.atoms) == 432

    #then lets find neighbors
    #cutoff method - first shell only
    sys.find_neighbors(method = 'cutoff', cutoff=0.867)
    #any atom should have 8 neighbors
    atoms = sys.atoms
    assert atoms[0].coordination == 8

    sys.reset_neighbors()

    #cutoff method - second shell
    sys.find_neighbors(method = 'cutoff', cutoff=1.1)
    #any atom should have 8 neighbors
    atoms = sys.atoms
    assert atoms[0].coordination == 14

    #cutoff method - third shell
    sys.find_neighbors(method = 'cutoff', cutoff=1.5)
    #any atom should have 8 neighbors
    atoms = sys.atoms
    assert atoms[0].coordination == 26

    sys.reset_neighbors()
    #voronoi method - first shell only
    sys.find_neighbors(method = 'voronoi')
    #any atom should have 8 neighbors
    atoms = sys.atoms
    assert atoms[0].coordination == 14

    assert np.round(sys.get_distance(atoms[0], atoms[1]), decimals=2) == 0.87


def test_neighbors_system_filter():
    #create some atoms
    atoms, boxdims = pcs.make_crystal('bcc', repetitions = [5, 5, 5])
    sys = pc.System()
    sys.box = boxdims
    sys.atoms = atoms


    sys.find_neighbors(method = 'cutoff', cutoff=0.867)
    #any atom should have 8 neighbors
    atoms = sys.atoms
    assert atoms[0].coordination == 8

    #now we take all the neighbors of atom0 and replace half of
    #them with a different type
    neighs = atoms[0].neighbors

    #replace the neighs
    atoms[neighs[0]].type = 2
    atoms[neighs[1]].type = 2

    #now set these atoms back
    #for atom in atoms:
    sys.set_atom(atoms[neighs[0]])
    sys.set_atom(atoms[neighs[1]])

    #recalculate neighbors with filter
    sys.find_neighbors(method = 'cutoff', cutoff=0.867, filter='type')
    #any atom should have 8 neighbors
    atoms = sys.atoms
    assert atoms[0].coordination == 6


def test_neighbors_diamond():
    atoms, box = pcs.make_crystal(structure="diamond", lattice_constant=5.67, repetitions=(6,6,6))
    sys = pc.System()
    sys.box = box
    sys.atoms = atoms

    sys.find_diamond_neighbors()
    sys.calculate_q([4,6], averaged=True)
    assert (np.mean(sys.get_qvals(4))-0.19094065395649326) < 1E-2
    assert (np.mean(sys.get_qvals(6))-0.5745242597140696) < 1E-2

    atoms, box = pcs.make_crystal(structure="fcc", lattice_constant=4.07, repetitions=(6,6,6))
    sys = pc.System()
    sys.box = box
    sys.atoms = atoms

    sys.find_neighbors(method="cutoff", cutoff=0)
    sys.calculate_q([4,6], averaged=True)
    assert (np.mean(sys.get_qvals(4))-0.19094065395649326) < 1E-2
    assert (np.mean(sys.get_qvals(6))-0.5745242597140696) < 1E-2

    atoms, box = pcs.make_crystal(structure="fcc", lattice_constant=4.07, repetitions=(6,6,6))
    sys = pc.System()
    sys.box = box
    sys.atoms = atoms

    sys.find_diamond_neighbors()
    sys.calculate_q([4,6], averaged=True)
    assert (np.mean(sys.get_qvals(4))-0.210287193019979) < 1E-2
    assert (np.mean(sys.get_qvals(6))-0.2350776015575979) < 1E-2    


import pytest
import numpy as np
import pyscal.core as pc
import pyscal.crystal_structures as pcs

def test_ordered_disorder():
    sys = pc.System()
    sys.read_inputfile('examples/conf.fcc.dump')
    sys.find_neighbors(method='cutoff', cutoff=0)
    sys.calculate_q(6)
    sys.calculate_disorder(averaged=True)
    atoms = sys.atoms
    disorder = [atom.disorder for atom in atoms]
    assert np.mean(disorder) < 0.50

    disorder = [atom.avg_disorder for atom in atoms]
    assert np.mean(disorder) < 0.50

def test_disordered_disorder():
    sys = pc.System()
    sys.read_inputfile('examples/conf.lqd')
    sys.find_neighbors(method='cutoff', cutoff=0)
    sys.calculate_q(6)
    sys.calculate_disorder(averaged=True)
    atoms = sys.atoms
    disorder = [atom.disorder for atom in atoms]
    assert np.mean(disorder) > 1.00

    disorder = [atom.avg_disorder for atom in atoms]
    assert np.mean(disorder) > 1.00



import pytest
import os
import numpy as np
import pyscal.core as pc
import pyscal.crystal_structures as pcs

def test_q_2():
    #this might take a while, it will find all qs
    atoms, boxdims = pcs.make_crystal('bcc', repetitions = [6, 6, 6])
    sys = pc.System()
    sys.box = boxdims
    sys.atoms = atoms

    #sys.read_inputfile("tests/bcc.dat")
    sys.find_neighbors(method = 'voronoi')
    atoms = sys.atoms
    vols = []
    avgvols = []
    for atom in atoms:
        vols.append(atom.volume)
        avgvols.append(atom.avg_volume)
    assert np.mean(np.array(vols)) == 0.5
    assert np.mean(np.array(avgvols)) == 0.5



import pytest
import os
import numpy as np
import pyscal.core as pc
import pyscal.crystal_structures as pcs

def test_sro():
    atoms, box = pcs.make_crystal('l12', lattice_constant=4.00, repetitions=[2,2,2])
    sys = pc.System()
    sys.box = box
    sys.atoms = atoms

    sys.find_neighbors(method='cutoff', cutoff=4.5)
    sro = sys.calculate_sro(reference_type=1, average=True)
    assert np.round(sro[0], decimals=2) == -0.33
    assert sro[1] == 1.0

    atoms = sys.atoms
    sro = atoms[4].sro
    assert np.round(sro[0], decimals=2) == -0.33
    assert sro[1] == 1.0

    sys.find_neighbors(method='cutoff', cutoff=4.5)
    sro = sys.calculate_sro(reference_type=1, average=True, shells=1)
    assert np.round(sro[0], decimals=2) == 0.11



import pytest
import os
import numpy as np
import pyscal.core as pc
from ase.build import bulk

def test_cubic():
    #this might take a while, it will find all qs
    atoms = bulk("Fe")
    sys = pc.System()
    sys.read_inputfile(atoms, format="ase")
    box, atoms = sys.extract_cubic_box()

    assert np.sum(np.array(atoms[0].pos)-np.array([0.0, 0.0, 0.0])) < 1E-10
    assert np.sum(np.array(atoms[1].pos)-np.array([1.435, 1.435, 1.435])) < 1E-10



import pytest
import os
import numpy as np
import pyscal.core as pc
import pyscal.crystal_structures as pcs
#from pytest.benchmark import benchmark
def test_neighbors_sann():
    #create some atoms
    atoms, boxdims = pcs.make_crystal('bcc', repetitions = [6, 6, 6])
    sys = pc.System()
    sys.box = boxdims
    sys.atoms = atoms


    #then lets find neighbors
    #SANN algo test
    #sys.find_neighbors(method = 'cutoff', cutoff='sann')
    sys.find_neighbors(method='cutoff', cutoff='sann')
    #any atom should have 8 neighbors
    atoms = sys.atoms
    assert atoms[0].coordination == 14

    sys.find_neighbors(method = 'cutoff', cutoff='sann', threshold=1)
    #any atom should have 8 neighbors
    atoms = sys.atoms
    assert atoms[0].coordination == 14

    sys.calculate_q(8, averaged=True)
    q = sys.get_qvals(8, averaged=True)
    assert np.round(np.mean(np.array(q), dtype=np.float64), decimals=2) == 0.45



def test_neighbors_adaptive():
    #create some atoms
    atoms, boxdims = pcs.make_crystal('bcc', repetitions = [6, 6, 6])
    sys = pc.System()
    sys.box = boxdims
    sys.atoms = atoms

    #then lets find neighbors
    #SANN algo test
    sys.find_neighbors(method = 'cutoff', cutoff='adaptive')
    #any atom should have 8 neighbors
    atoms = sys.atoms
    assert atoms[0].coordination == 14
